@@ -836,6 +836,15 @@ compare_spct(filter_mspct(list(pet = polyester.spct,
              w.band = 50,
             .comparison.fun = `<`)
 
+## -----------------------------------------------------------------------------
+illuminance(sun.spct)
+
+## -----------------------------------------------------------------------------
+illuminance(sun.spct, std = "CIE10deg")
+
+## -----------------------------------------------------------------------------
+illuminance(sun.daily.spct)
+
 ## -----------------------------------------------------------------------------
 color_of(550) # green
 color_of(630) # red
